﻿cDisplay = 
{  
    ---
    -- A shortcut to add a message to the default chat window
    printStyle = {
        white = { r=1.0, g=1.0, b=1.0, group=54, addToStart=false },
        gray = { r=0.5, g=0.5, b=0.5, group=53, addToStart=false }
    },
    Print = function(self, message, style, color)
		local r, g, b = unpack(color or {1, 1, 1});
        if style == nil then
            style = self.printStyle.white
        end
        DEFAULT_CHAT_FRAME:AddMessage(message, r, g, b, style.group, style.addToStart);
    end,
	
	Debug = function(self, message)
		if sConfig.general.showDebug then
			self:Print(message, self.printStyle.gray, {0.5, 0.5, 0.5})
		end
	end,

    ---
    -- Average Window controls
    ---
    Average = 
    {
        window = nil,
        lines = {},
        backdrop = nil,
    
        -- Window Style
        visible = true, -- Note that this should be treated as a private member. Use the SetVisibility function.
        showBackground = true,
        textMargin = 5,
        lineSpacing = 2,
        
        ---
        -- Creates the window
        Initialize = function(self)
            if self.window ~= nil then
                return
            end
        
            self.window = XToLevelDisplay
            self.backdrop = self.window:GetBackdrop();
            
            -- Create the lines
            self.lines.header = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_h1')
            self.lines.header:SetText('XToLevel')
            self.lines.header.group = "header"
            self.lines.header.tabIndex = 1
            
            self.lines.playerKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerKills:SetText(L["Kills"] .. ': ...')
            self.lines.playerKills.group = "player"
            self.lines.playerKills.tabIndex = 2
            
            self.lines.playerQuests = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerQuests:SetText(L["Quests"] .. ': ...')
            self.lines.playerQuests.group = "player"
            self.lines.playerQuests.tabIndex = 3
			
			self.lines.playerBGs = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerBGs:SetText(L["Battlegrounds"] .. ': ...')
            self.lines.playerBGs.group = "player"
            self.lines.playerBGs.tabIndex = 4
			
			self.lines.playerBGOs = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerBGOs:SetText(L["Battleground Objectives Short"] .. ': ...')
            self.lines.playerBGOs.group = "player"
            self.lines.playerBGOs.tabIndex = 5
			
			self.lines.petKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.petKills:SetText(L["Pet"] .. ': ...')
            self.lines.petKills.group = "player"
            self.lines.petKills.tabIndex = 6
            
            self:Update()
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the window
        Update = function(self)
            if self.window == nil then
                return
            end
            
            if not sConfig.averageDisplay.visible then
                self.window:Hide()
                return
            elseif not cPlayer.isActive and not (cPet.isActive or cPet.hasBeenActive) then
				self.window:Hide()
                return
			else
				self.window:Show()
            end
			
			if sConfig.averageDisplay.header then
				self.lines.header:Show()
			end
			
			if not cPlayer.isActive then
				self.lines.playerKills:Hide()
				self.lines.playerQuests:Hide()
			else
				self.lines.playerKills:Show()
				self.lines.playerQuests:Show()
			end
			if not (cPet.isActive or cPet.hasBeenActive) then
				self.lines.petKills:Hide()
			else
				self.lines.petKills:Show()
			end
			if not cPlayer.isActive or (cPlayer:GetAverageBGsRemaining() == nil or not ShowBattlegroundData()) then
				self.lines.playerBGs:Hide()
			else
				self.lines.playerBGs:Show()
			end
			if not cPlayer.isActive or (cPlayer:GetAverageBGObjectivesRemaining() == nil or not ShowBattlegroundData()) then
				self.lines.playerBGOs:Hide()
			else
				self.lines.playerBGOs:Show()
			end
			
            -- Show / Hide the lines and update the anchors
            -- The elements are re-arraged into a temp array according to their tabIndex
            local iLines = {}
            for eName, eElem in pairs(self.lines) do
                iLines[eElem.tabIndex] = { name=eName, elem=eElem }
            end
            
            local nextAnchor = self.window
            local nextMarginV = self.textMargin
            local nextMarginH = self.textMargin
            local nextPoint = "TOPLEFT"
            local currentIndent =  -nextMarginH
            for index, value in ipairs(iLines) do
                local name = value.name
                local elem = value.elem
                
                elem:ClearAllPoints()
                if sConfig.averageDisplay[name] and elem:IsShown() then
                    -- Clear text indent if the previous element was in the same group
                    if nextAnchor.group == elem.group then
                        nextMarginH = 0
                    else
                        currentIndent = currentIndent + nextMarginH
                    end
                    elem:SetPoint("TOPLEFT", nextAnchor, nextPoint, nextMarginH, -nextMarginV)
                    elem.lineIndent = currentIndent
                    
                    nextAnchor = elem
                    nextMarginV = self.lineSpacing
                    nextPoint = "BOTTOMLEFT"
                else
                    elem:Hide()
                end
            end
            
            -- Update messages for the visible elements
            if self.lines.header:IsShown() then
                self.lines.header:SetText('XToLevel')
            end
			
			local labelColor, playerColor, petColor, playerProgress, petProgress;
			labelColor = "ffffff"
			
			if cPlayer.isActive then
				playerProgress = round(cPlayer.currentXP / cPlayer.maxXP * 100, 1)
				if sConfig.averageDisplay.colorText then
					playerProgressColor = GetProgressColor(playerProgress)
				end
				if self.lines.playerKills:IsShown() then
					if sConfig.averageDisplay.colorText then
						self.lines.playerKills:SetText("|cFF".. labelColor .. ((sConfig.averageDisplay.verbose and L["Kills"]) or L["Kills Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageKillsRemaining() .."|r")
					else
						self.lines.playerKills:SetText(((sConfig.averageDisplay.verbose and L["Kills"]) or L["Kills Short"]) .. ': ' .. cPlayer:GetAverageKillsRemaining())
					end
				end
				if self.lines.playerQuests:IsShown() then
					if sConfig.averageDisplay.colorText then
						self.lines.playerQuests:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Quests"]) or L["Quests Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageQuestsRemaining() .."|r")
					else
						self.lines.playerQuests:SetText(((sConfig.averageDisplay.verbose and L["Quests"]) or L["Quests Short"]) .. ': ' .. cPlayer:GetAverageQuestsRemaining())
					end
				end
				if self.lines.playerBGs:IsShown() and cPlayer:GetAverageBGsRemaining() ~= nil then
					if sConfig.averageDisplay.colorText then
						self.lines.playerBGs:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Battlegrounds"]) or L["Battlegrounds Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageBGsRemaining() .."|r")
					else
						self.lines.playerBGs:SetText(((sConfig.averageDisplay.verbose and L["Battlegrounds"]) or L["Battlegrounds Short"]) .. ': ' .. cPlayer:GetAverageBGsRemaining())
					end
				end
				if self.lines.playerBGOs:IsShown() then -- and cPlayer:GetAverageBGObjectivesRemaining() ~= nil
					if sConfig.averageDisplay.colorText then
						self.lines.playerBGOs:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Battleground Objectives"]) or L["Battleground Objectives Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageBGObjectivesRemaining() .."|r")
					else
						self.lines.playerBGOs:SetText(((sConfig.averageDisplay.verbose and L["Battleground Objectives"]) or L["Battleground Objectives Short"]) .. ': ' .. cPlayer:GetAverageBGObjectivesRemaining())
					end
				end
			end
			
			if cPet.isActive and self.lines.petKills:IsShown() then
				petProgress = round(cPet.xp / cPet.maxXP * 100, 1)
				if sConfig.averageDisplay.colorText then
					petProgressColor = GetProgressColor(petProgress)
					self.lines.petKills:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Pet"]) or L["Pet Short"]) .. ':|r |cFF'.. petProgressColor .. cPet:GetAverageKillsRemaining() .."|r")
				else
					self.lines.petKills:SetText(((sConfig.averageDisplay.verbose and L["Pet"]) or L["Pet Short"]) .. ': ' .. cPet:GetAverageKillsRemaining())
				end
			end
            
            -- Calculate the height and width of the lines
            local maxWidth = 0
            local totalHeight = self.textMargin * 2
            for name, value in pairs(self.lines) do
                if value:IsVisible() then
                    local currentWidth = value:GetWidth() + (value.lineIndent or 0)
                    if currentWidth  > maxWidth then
                        maxWidth = currentWidth
                    end
                    totalHeight = totalHeight + value:GetHeight() + self.lineSpacing
                end
            end
            local totalWidth = maxWidth + (self.textMargin * 2)
            
            -- Set the display box size
            self.window:SetWidth(totalWidth)
            self.window:SetHeight(totalHeight)
            
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the backdrop
        updateBackdrop = function(self)
            if self.window == nil then
                return
            end
            
            if sConfig.averageDisplay.backdrop then
                self.window:SetBackdrop(self.backdrop);
                self.window:SetBackdropColor("0.0", "0.75", "0.5", "0.75");
                self.window:SetBackdropBorderColor("0.0", "0.0", "0.0", "1.0");
            else
                self.window:SetBackdrop(nil);
            end
        end,
        
        -- Show / Hide the window
        ToggleVisibility = function(self, newValue)
            if self.window == nil then
                return
            end
        
            if(newValue) then
                self.visible = newValue
            else
                self.visible = not self.visible
            end
            
            if self.visible then
                self.window:Show()
            else
                self.window:Hide()
            end
        end
    },
    
    ---
    -- Controls for the floating display
    -- The settings check is added here to simplify the calling code.
    ---
    Floating = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5 },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, fade=6 },
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerFloating then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                self:Print(message, sConfig.messages.colors.playerKill, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petFloating then
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                self:Print(message, sConfig.messages.colors.petKill, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerFloating then
                local message = questsRequired .. L["Quests Needed"]
                self:Print(message, sConfig.messages.colors.playerQuest, self.questStyle)
            end
        end,
		
		PrintBattleground = function(self, bgsRequired)
            if sConfig.messages.playerFloating then
                local message = bgsRequired .. L["Battlegrounds Needed"]
                self:Print(message, sConfig.messages.colors.playerBattleground, self.questStyle)
            end
        end,
		PrintBGObjective = function(self, bgsRequired)
            if sConfig.messages.playerFloating and sConfig.messages.bgObjectives then
                local message = bgsRequired .. L["Battleground Objectives Needed"]
                self:Print(message, sConfig.messages.colors.playerBattleground, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerFloating then
                local message = L["Level Reached"]
                self:Print(message, sConfig.messages.colors.playerLevel, self.levelStyle)
            end
        end,
        
        Print = function(self, text, color, style)
			local r, g, b = unpack(color)
            UIErrorsFrame:AddMessage(text, r, g, b, style.group, style.fade);
        end
    },
    
    ---
    -- Controls for the chat display
    ---
    Chat = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5, addToStart=false },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, addToStart=false},
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerChat then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                cDisplay:Print(message, self.killStyle, sConfig.messages.colors.playerKill)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petChat then
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                cDisplay:Print(message, self.petStyle, sConfig.messages.colors.petKill)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerChat then
                local message = questsRequired .. L["Quests Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerQuest)
            end
        end,
		
		PrintBattleground = function(self, bgsRequired)
            if sConfig.messages.playerChat then
                local message = bgsRequired .. L["Battlegrounds Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerBattleground)
            end
        end,
		PrintBGObjective = function(self, bgsRequired)
            if sConfig.messages.playerChat and sConfig.messages.bgObjectives then
                local message = bgsRequired .. L["Battleground Objectives Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerBattleground)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerChat then
                local message = L["Level Reached"]
                cDisplay:Print(message, self.levelStyle, sConfig.messages.colors.playerLevel)
            end
        end
    },
    
    ---
    -- Controls for the configuration window
    ---
    Config =
    {
        waterfall = nil,
		
		Initialize = function(self)
			self.waterfall = AceLibrary("Waterfall-1.0")
            self.waterfall:Register('ConfigWindow',
				'title',L["Config Header"],
 				'aceOptions', self:getOptions(),
				'treeLevels', 3,
				'colorR', 0.0,
                'colorG', 0.5,
                'colroB', 0.25
			);
		end,
		
		Open = function(self, page)
			if page == nil then page = "general" end
			self.waterfall:Open('ConfigWindow', page)
		end,
		
		Refresh = function(self, page)
			self.waterfall:Close('ConfigWindow')
			self:Initialize()
			self:Open(page or "general")
		end,
		
		getOptions = function(self)
			return {
				type = 'group',
				name = L["XToLevel"],
				desc = L["Config Header"],
				order = 1,
				args = {
					general = {
						type = 'group',
						name = L["General Name"],
						order = 100,
						args = {
							localeHeader = {
								type = 'header',
								name = L["Locale Header"],
								order = 101,
							},
							localeDropdown = {
								type = 'text',
								name = L["Display Locale"],
								get = function() return sConfig.general.displayLocale or GetLocale(); end,
								set = function(value)
									LOCALE_DISPLAY = value
									L = LOCALE[LOCALE_DISPLAY]
									sConfig.general.displayLocale = value
									
									cDisplay.Config:Refresh("general")
									cDisplay.Average:Update()
								end,
								validate = {"enUS", "frFR", "deDE", "esES"},
								order = 102,
							},
							showDebugToogle = {
								type = 'toggle',
								name = L["Show Debug Info"],
								desc = 'Displayes various debug info in the default chat window. Meant only as a development tool.',
								get = function() return sConfig.general.showDebug end,
								set = function(x) sConfig.general.showDebug = x end,
								order = 103,
							},
						},
					},
					messages = {
						type = 'group',
						name = L["Messages Name"],
						order = 200,
						args = {
							messagesPlayerHeader = {
								type = 'header',
								name = L["Player Messages"],
								order = 201,
							},
							messagesPlayerFloating = {
								type = 'toggle',
								name = L["Show Floating"],
								get = function() return sConfig.messages.playerFloating end,
								set = function(x) sConfig.messages.playerFloating = x end,
								order = 202,
							},
							messagesPlayerChat = {
								type = 'toggle',
								name = L["Show In Chat"],
								get = function() return sConfig.messages.playerChat end,
								set = function(x) sConfig.messages.playerChat = x end,
								order = 203,
							},
							messagesBGObjectives = {
								type = 'toggle',
								name = L["Show BG Objectives"],
								get = function() return sConfig.messages.bgObjectives end,
								set = function(x) sConfig.messages.bgObjectives = x end,
								order = 204,
							},
							
							messagesMargin1 = {
								type = 'header',
								name = ' ',
								order = 210,
							},
							messagesPetHeader = {
								type = 'header',
								name = L["Pet Messages"],
								order = 211,
							},
							messagesPetFloating = {
								type = 'toggle',
								name = L["Show Floating"],
								get = function() return sConfig.messages.petFloating end,
								set = function(x) sConfig.messages.petFloating = x end,
								order = 212,
							},
							messagesPetChat = {
								type = 'toggle',
								name = L["Show In Chat"],
								get = function() return sConfig.messages.petChat end,
								set = function(x) sConfig.messages.petChat = x end,
								order = 213,
							},
							
							messagesMargin2 = {
								type = 'header',
								name = ' ',
								order = 220,
							},
							messagesColorHeader = {
								type = 'header',
								name = L['Message Colors'],
								order = 221,
							},
							playerKillMessageColor = {
								type = 'color',
								name = L['Player Kill Color'],
								get = function() 
									return unpack(sConfig.messages.colors.playerKill); 
								end,
								set = function(r, g, b, a)
									sConfig.messages.colors.playerKill = {r, g, b, a};
								end,
								order = 222,
							},
							playerQuestMessageColor = {
								type = 'color',
								name = L['Player Quest Color'],
								get = function() 
									return unpack(sConfig.messages.colors.playerQuest); 
								end,
								set = function(r, g, b, a)
									sConfig.messages.colors.playerQuest = {r, g, b, a};
								end,
								order = 223,
							},
							playerBGMessageColor = {
								type = 'color',
								name = L['Player Battleground Color'],
								get = function() 
									return unpack(sConfig.messages.colors.playerBattleground); 
								end,
								set = function(r, g, b, a)
									sConfig.messages.colors.playerBattleground = {r, g, b, a};
								end,
								order = 224,
							},
							playerLevelMessageColor = {
								type = 'color',
								name = L['Player Level Color'],
								get = function() 
									return unpack(sConfig.messages.colors.playerLevel); 
								end,
								set = function(r, g, b, a)
									sConfig.messages.colors.playerLevel = {r, g, b, a};
								end,
								order = 225,
							},
							petKillMessageColor = {
								type = 'color',
								name = L['Pet Kill Color'],
								get = function() 
									return unpack(sConfig.messages.colors.petKill); 
								end,
								set = function(r, g, b, a)
									sConfig.messages.colors.petKill = {r, g, b, a};
								end,
								order = 226,
							},
							messageColorReset = {
								type = 'execute',
								name = L['Color Reset'],
								desc = L['Color Reset Description'],
								func = function()
									StaticPopup_Show("Config_MessageColorsReset");
								end,
								disabled = function()
									return false;
								end,
								order = 227,
							},
							
						},
					},
					average = {
						type = 'group',
						name = L["Average Name"],
						order = 300,
						args = {
							averageAppearenceHeader = {
								type = 'header',
								name = L["Appearence Header"],
								order = 301,
							},
							averageVisible = {
								type = 'toggle',
								name = L["Show Average Window"],
								get = function() return sConfig.averageDisplay.visible end,
								set = function(x) sConfig.averageDisplay.visible = x; cDisplay.Average:Update(); end,
								order = 302,
							},
							averageShowHeader = {
								type = 'toggle',
								name = L["Show XToLevel Header"],
								get = function() return sConfig.averageDisplay.header end,
								set = function(x) sConfig.averageDisplay.header = x; cDisplay.Average:Update(); end,
								order = 303,
							},
							averageShowBackdrop = {
								type = 'toggle',
								name = L["Show Window Frame"],
								get = function() return sConfig.averageDisplay.backdrop end,
								set = function(x) sConfig.averageDisplay.backdrop = x; cDisplay.Average:Update(); end,
								order = 304,
							},
							averageShowVerbose= {
								type = 'toggle',
								name = L["Show Verbose Text"],
								get = function() return sConfig.averageDisplay.verbose end,
								set = function(x) sConfig.averageDisplay.verbose = x; cDisplay.Average:Update(); end,
								order = 304,
							},
							averageShowColor= {
								type = 'toggle',
								name = L["Show Colored Text"],
								get = function() return sConfig.averageDisplay.colorText end,
								set = function(x) sConfig.averageDisplay.colorText = x; cDisplay.Average:Update(); end,
								order = 304,
							},
							averageShowTooltip= {
								type = 'toggle',
								name = L["Show Tooltip"],
								get = function() return sConfig.averageDisplay.tooltip end,
								set = function(x) sConfig.averageDisplay.tooltip = x; end,
								order = 304,
							},
							marginHeader2 = {
								type = 'header',
								name = "",
								order = 320,
							},
							averageBehaviorHeader = {
								type = 'header',
								name = L["Behavior Header"],
								order = 321,
							},
							averageBehaviorLock = {
								type = 'toggle',
								name = L["Lock Avarage Display"],
								get = function() return not sConfig.general.allowDrag end,
								set = function(x) sConfig.general.allowDrag = not x; cDisplay.Average:Update(); end,
								order = 322,
							},
							averageBehaviorAllowClick = {
								type = 'toggle',
								name = L["Allow Click"],
								get = function() return sConfig.general.allowSettingsClick end,
								set = function(x) sConfig.general.allowSettingsClick = x; cDisplay.Average:Update(); end,
								order = 323,
							},
							marginHeader1 = {
								type = 'header',
								name = "",
								order = 340,
							},
							averageDataHeader = {
								type = 'header',
								name = L["Data Header"],
								order = 341,
							},
							averageShowKills = {
								type = 'toggle',
								name = L["Show Kills"],
								get = function() return sConfig.averageDisplay.playerKills end,
								set = function(x) sConfig.averageDisplay.playerKills = x; cDisplay.Average:Update(); end,
								order = 342,
							},
							averageShowQuests = {
								type = 'toggle',
								name = L["Show Quests"],
								get = function() return sConfig.averageDisplay.playerQuests end,
								set = function(x) sConfig.averageDisplay.playerQuests = x; cDisplay.Average:Update(); end,
								order = 343,
							},
							averageShowBattlegroundObjectives = {
								type = 'toggle',
								name = L["Show Battleground Objectives"],
								get = function() return sConfig.averageDisplay.playerBGOs end,
								set = function(x) sConfig.averageDisplay.playerBGOs = x; cDisplay.Average:Update(); end,
								order = 344,
							},
							averageShowBattlegrounds = {
								type = 'toggle',
								name = L["Show Battlegrounds"],
								get = function() return sConfig.averageDisplay.playerBGs end,
								set = function(x) sConfig.averageDisplay.playerBGs = x; cDisplay.Average:Update(); end,
								order = 345,
							},
							averageShowPet = {
								type = 'toggle',
								name = L["Show Pet Kills"],
								get = function() return sConfig.averageDisplay.petKills end,
								set = function(x) sConfig.averageDisplay.petKills = x; cDisplay.Average:Update(); end,
								order = 346,
							},
							marginHeader2 = {
								type = 'header',
								name = "",
								order = 360,
							},
							averageBehaviorHeader = {
								type = 'header',
								name = 'Numer of records used',
								order = 361,
							},
							averagePlayerKillRecords = {
								type = 'range',
								min = 1,
								max = 100,
								step = 1,
								name = 'Player kills',
								get = function() return sConfig.averageDisplay.playerKillListLength; end,
								set = function(value) 
									local nValue = tonumber(value)
									if nValue > 0 then
										sConfig.averageDisplay.playerKillListLength = value;
										cPlayer.killAverage = null;
										cDisplay.Average:Update();
									end
								end,
								order = 362,
							},
							averagePlayerQuestRecords = {
								type = 'range',
								min = 1,
								max = 100,
								step = 1,
								name = 'Player quests',
								get = function() return sConfig.averageDisplay.playerQuestListLength; end,
								set = function(value) 
									local nValue = tonumber(value)
									if nValue > 0 then
										sConfig.averageDisplay.playerQuestListLength = value;
										cPlayer.questAverage = null;
										cDisplay.Average:Update();
									end
								end,
								order = 363,
							},
							averagePlayerBGRecords = {
								type = 'range',
								name = 'Player BGs',
								min = 1,
								max = 100,
								step = 1,
								get = function() return sConfig.averageDisplay.playerBGListLength; end,
								set = function(value) 
									local nValue = tonumber(value)
									if nValue > 0 then
										sConfig.averageDisplay.playerBGListLength = value;
										cPlayer.bgAverage = null;
										cPlayer.bgObjAverage = null;
										cDisplay.Average:Update();
									end
								end,
								order = 364,
							},
							averagePetKillRecords = {
								type = 'range',
								name = 'Pet Kills',
								min = 1,
								max = 100,
								step = 1,
								get = function() return sConfig.averageDisplay.petKillListLength; end,
								set = function(value) 
									local nValue = tonumber(value)
									if nValue > 0 then
										sConfig.averageDisplay.petKillListLength = value;
										cPet.killAverage = null;
										cDisplay.Average:Update();
									end
								end,
								order = 365,
							},
						},
					},
					ldb = {
						type = 'group',
						name = L["LDB Name"],
						order = 400,
						args = {
							ldbGeneralHeader = {
								type = 'header',
								name = L["LDB General Header"],
								order = 401,
							},
							ldbGeneralShowText = {
								type = 'toggle',
								name = L["Show LDB Text"],
								get = function() return sConfig.ldb.showText end,
								set = function(x) sConfig.ldb.showText = x; UpdateDataObject(); end,
								order = 402,
							},
							ldbGeneralShowLabel = {
								type = 'toggle',
								name = L["Show LDB Label"],
								get = function() return sConfig.ldb.showLabel end,
								set = function(x) sConfig.ldb.showLabel = x; UpdateDataObject(); end,
								order = 403,
							},
							ldbGeneralShowIcon = {
								type = 'toggle',
								name = L["Show LDB Icon"],
								get = function() return sConfig.ldb.showIcon end,
								set = function(x) sConfig.ldb.showIcon = x; UpdateDataObject(); end,
								order = 405,
							},
							ldbAllowTextColor = {
								type = 'toggle',
								name = L["LDB Allow Text Color"],
								get = function() return sConfig.ldb.allowTextColor end,
								set = function(x) sConfig.ldb.allowTextColor = x; UpdateDataObject(); end,
								order = 406,
							},
							marginHeader1 = {
								type = 'header',
								name = "",
								order = 409,
							},
							ldbTextHeader = {
								type = 'header',
								name = L["LDB Text Header"],
								order = 410,
							},
							ldbTextPattern = {
								type = 'text',
								name = L["LDB Text Pattern"],
								get = function() return sConfig.ldb.textPattern; end,
								set = function(value) 
									sConfig.ldb.textPattern = value; UpdateDataObject(); 
									cDisplay.Config:Refresh("ldb")
								end,
								validate = {"default", "minimal", "minimal_dashed", "brackets", "custom"},
								order = 411,
							},
							ldbTextCustomPattern = {
								type = 'text',
								name = '',
								disabled = function() return sConfig.ldb.textPattern ~= "custom"; end,
								get = function() return sData.customPattern; end,
								set = function(value) 
									sData.customPattern = value; UpdateDataObject(); 
								end,
								order = 412,
							},
							ldbTextVerbose = {
								type = 'toggle',
								name = L["LDB Text Verbose"],
								disabled = function() return sConfig.ldb.textPattern ~= "default"; end,
								get = function() return sConfig.ldb.text.verbose; end,
								set = function(x) sConfig.ldb.text.verbose = x; UpdateDataObject(); end,
								order = 413,
							},
							ldbTextUseColorValues = {
								type = 'toggle',
								name = L["LDB Text Value Colors"],
								get = function() return sConfig.ldb.text.colorValues end,
								set = function(x) sConfig.ldb.text.colorValues = x; UpdateDataObject(); end,
								order = 414,
							},
							marginHeader2 = {
								type = 'header',
								name = "",
								order = 415,
							},
							ldbTextKills = {
								type = 'toggle',
								name = L["Text Show Kills"],
								get = function() return sConfig.ldb.text.kills end,
								set = function(x) sConfig.ldb.text.kills = x; UpdateDataObject(); end,
								order = 416,
							},
							ldbTextQuests = {
								type = 'toggle',
								name = L["Text Show Quests"],
								get = function() return sConfig.ldb.text.quests end,
								set = function(x) sConfig.ldb.text.quests = x; UpdateDataObject(); end,
								order = 417,
							},
							ldbTextBattlegrounds = {
								type = 'toggle',
								name = L["Text Show Battlegrounds"],
								get = function() return sConfig.ldb.text.bgs end,
								set = function(x) sConfig.ldb.text.bgs = x; UpdateDataObject(); end,
								order = 418,
							},
							ldbTextBattlegroundObjectives = {
								type = 'toggle',
								name = L["Show Battleground Objectives"],
								get = function() return sConfig.ldb.text.bgo end,
								set = function(x) sConfig.ldb.text.bgo = x; UpdateDataObject(); end,
								order = 418,
							},
							ldbTextXP = {
								type = 'toggle',
								name = L["Text Show XP Progress"],
								get = function() return sConfig.ldb.text.xp end,
								set = function(x) sConfig.ldb.text.xp = x; UpdateDataObject(); end,
								order = 419,
							},
							ldbTextPet = {
								type = 'toggle',
								name = L["Text Show Pets"],
								get = function() return sConfig.ldb.text.pet end,
								set = function(x) sConfig.ldb.text.pet = x; UpdateDataObject(); end,
								order = 420,
							},
							ldbTextPetXP = {
								type = 'toggle',
								name = L["Text Show Pet XP Progress"],
								get = function() return sConfig.ldb.text.petxp end,
								set = function(x) sConfig.ldb.text.petxp = x; UpdateDataObject(); end,
								order = 421,
							},
							
							marginHeader3 = {
								type = 'header',
								name = "",
								order = 424,
							},
						},
					},
					tooltip = {
						type = 'group',
						name = L["LDB Tooltip Header"],
						order = 500,
						args = {
							ldbTooltipDetails = {
								type = 'toggle',
								name = L["Show Tooltip Details"],
								get = function() return sConfig.ldb.tooltip.showDetails end,
								set = function(x) sConfig.ldb.tooltip.showDetails = x; UpdateDataObject(); end,
								order = 502,
							},
							ldbTooltipExperience = {
								type = 'toggle',
								name = L["Show Tooltip Experience"],
								get = function() return sConfig.ldb.tooltip.showExperience end,
								set = function(x) sConfig.ldb.tooltip.showExperience = x; UpdateDataObject(); end,
								order = 503,
							},
							ldbTooltipBattleground = {
								type = 'toggle',
								name = L["Show Tooltip Battleground"],
								get = function() return sConfig.ldb.tooltip.showBGInfo end,
								set = function(x) sConfig.ldb.tooltip.showBGInfo = x; UpdateDataObject(); end,
								order = 504,
							},
						},
					},
				},
			}
		end,
    }
}
